@@ -191,27 +191,44 @@ class WiringInstruction (object): # <<<
             qubitsmap [j] = i
             assert i not in qubitsmap_forward
             qubitsmap_forward [i] = j
-        ioffset = 0
+        ##
+        # The requirements for qc1 to be wireable to qc0.
+        #
+        # (In sum: Any unwired qubits/clbits must belong in qc1's registers,
+        #          in which no wired qubits/clbits are allowed.)
+        #
+        # 1. After all qubits with indices as registered in qubitsmap are
+        #    taken into account, any remaining qubits must belong in quantum
+        #    registers.
+        # 2. In any of those remaining registers, there should not exist any
+        #    wired qubit.  Namely, those registers should consist of purely
+        #    unwired qubits.
+        # 3. At the moment, no classical bits are allowed to be wired, and
+        #    all classical bits must be accounted for precisely by all
+        #    classical registers of qc1.
+        ##
+        unwired_qbits_map = dict ((qb, i) for i, qb in enumerate (qc1.qubits)
+                                   if i not in qubitsmap)
         qregs2add = []
         for r in qc1.qregs:
+            test = sum (b in unwired_qbits_map for b in r)
+            if test == r.size:
+                qregs2add.append (r)
+            elif test:
+                raise TypeError ("Wiring is implemented only if unwired "
+                                 "qubits are not mixed with wired qubits "
+                                 "in their common register.")
+        for r in qregs2add:
             n = r.size
-            test = sum (i in qubitsmap for i in range (ioffset, ioffset + n))
-            if not test: # all out
-                qregs2add.append ((r, ioffset))
-            elif test != n: # partially in
-                raise TypeError ("Wiring is implemented per quantum register "
-                                 "only---partially wired quantum register "
-                                 "is not allowed yet.")
-            ioffset += n
-        for r, ioffset in qregs2add:
-            n = r.size
-            for i in range (ioffset, ioffset + n):
-                assert i not in qubitsmap
-                qubitsmap [i] = n0 + i - ioffset
+            ioffset = unwired_qbits_map [r [0]]
+            # anticipate that qregs will be added to qc0
+            for i in range (n):
+                assert i + ioffset not in qubitsmap
+                qubitsmap [i + ioffset] = n0 + i
             n0 += n
         if len (qubitsmap) != n1:
-            raise TypeError ("Wiring is limited to a circuit whose qubits "
-                             "are completely accounted for by its qregs.")
+            raise TypeError ("Wiring is limited to a circuit whose unwired "
+                             "qubits are totally accounted for by its qregs.")
         ##
         # At this point, qbitsmap is complete and it contains wired and
         # non-wired indices of qc1 as its keys, and no other keys.  Namely,
@@ -233,7 +250,7 @@ class WiringInstruction (object): # <<<
         # Adding registers can fail, if name conflict exists.  We leave it to
         # user to avoid any name collision.
         ##
-        for r, _ in qregs2add:
+        for r in qregs2add:
             qc0.add_register (r)
         for r in qc1.cregs:
             qc0.add_register (r)
@@ -246,19 +263,16 @@ def backends (provider = None, hub = 'ibm-q', backend = None, ** kwargs): # <<<
     """
     Yields backends.
 
-    If ``backend`` is already specified as a
-    :class:`~qiskit.providers.backend.Backend` instance, or an interable of
-    them, then they are yielded without having to use the provider
-    information, defined by ``provider`` and ``hub`` (see
-    :func:`~get_provider`).
+    Any :class:`~qiskit.providers.backend.Backend` instance, as specified as
+    the value of ``backend`` or any part of iterated values of ``backend``
+    will be left untouched and be yielded as is.  None of the arguments are
+    used in this case.
 
     If ``backend`` is left unspecified, is a string, or an iterable that
     yields a string among others, then the provider is necessasry to identify
-    and it will be obtained using ``provider`` and ``hub``.
-
-    If and when the provider is needed and its ``backends`` method is called,
-    ``kwargs`` will be passed to the method.  For example, kwargs can specify
-    a ``filters`` function.
+    and it will be obtained using ``provider`` and ``hub``.  Then, its
+    ``backends`` method is called with ``kwargs`` passed to it.  For example,
+    kwargs can specify a ``filters`` function.
 
     :param backend:  A string (backend name),
         a :class:`~qiskit.providers.backend.Backend` instance, an iterable of
@@ -294,14 +308,16 @@ def backends (provider = None, hub = 'ibm-q', backend = None, ** kwargs): # <<<
 # >>>
 def gather_counts (r, converter = None): # <<<
     """
-    Gathers counts from the run result (``r``).
+    Gathers counts from a run result (``r``).
 
     With ``converter`` unspecified, this function is just the same as
     ``r.get_counts ()``.
 
-    With ``converter`` specified, this function will look at the raw data
-    (``r.get_memory ()``) and applies ``converter`` to the raw data before
-    building the counter dictionary.
+    With ``converter`` specified, in which case it must be a callable, this
+    function will retrieve the raw data (``r.get_memory ()``) first (and so
+    the circuit/experiment must have been run with memory requested), apply
+    ``converter`` to each element of the raw data, and build the counter
+    dictionary based on the converted values.
     """
     if converter is None:
         return r.get_counts ()
@@ -310,6 +326,10 @@ def gather_counts (r, converter = None): # <<<
     return Counter (e for e in (converter (k) for k in m) if e is not None)
 # >>>
 def get_provider (provider = None, hub = 'ibm-q'): # <<<
+    """
+    If ``provider`` is not ``None``, then this function just returns it.
+    Otherwise, it will provision the provider based on ``hub``.
+    """
     if provider is None:
         from qiskit import IBMQ # pylint: disable=W0406,E0611
         provider = IBMQ.get_provider (hub = hub)
